//--------------------------------------------------------------------------------------------------
//	iseGPEMF.hxx
//	I. Nakagawa
//	2013/10/01
//--------------------------------------------------------------------------------------------------
#ifndef	ISE_GetPathEMF_hxx
#define	ISE_GetPathEMF_hxx

#include	<string>
#include	<cstdlib>
#include	<Windows.h>
#include	<tchar.h>

typedef	std::basic_string <TCHAR,std::char_traits<TCHAR>,std::allocator<TCHAR> >	tstring ;

//--------------------------------------------------------------------------------------------------
inline	tstring	ise_GetLongPathName	(const tstring& fileName)
{
	tstring	longPath = fileName ;
	size_t	size = MAX_PATH ;
	longPath.resize(size,0) ;
	DWORD	len  = ::GetLongPathName(fileName.c_str(),&longPath[0],DWORD(size)) ;
	if (len == 0) {	return	fileName ;	}
	return	longPath ;
	}

inline	tstring	ise_GetTempPath	(void)
{
	tstring	tmpName ;
	size_t	size = MAX_PATH ;
	tmpName.resize(size,0) ;
	::GetTempPath	(size,&tmpName[0]) ;
	tmpName = ::ise_GetLongPathName(tmpName) ;
	return	tmpName.c_str() ;
	}

//--------------------------------------------------------------------------------------------------
inline	tstring	ise_GetFileDir (const tstring& pathName)
{
	tstring	fileDrv ;	fileDrv.resize(MAX_PATH,0) ;
	tstring	fileDir ;	fileDir.resize(MAX_PATH,0) ;
	#if	(_MSC_VER >= 1400)
	_tsplitpath_s	(pathName.c_str(),	&fileDrv[0],MAX_PATH,	&fileDir[0],MAX_PATH,	NULL,0,	NULL,0) ;
	#else
	_tsplitpath		(pathName.c_str(),	&fileDrv[0],			&fileDir[0],			NULL,	NULL) ;
	#endif
	{	//	delete last '\\'
		tstring	tmp = fileDir.c_str() ;
		if (tmp.length() > 1) {
			if (tmp[tmp.length()-1] == _T('\\')) {
				fileDir = tmp.substr(0,tmp.length()-1) ;
				}
			}
		}
	return	tstring(fileDrv.c_str())+fileDir.c_str() ;
	}

inline	tstring	ise_GetFileName (const tstring& pathName)
{
	tstring	fileTtl ;	fileTtl.resize(MAX_PATH,0) ;
	tstring	fileExt ;	fileExt.resize(MAX_PATH,0) ;
	#if	(_MSC_VER >= 1400)
	_tsplitpath_s	(pathName.c_str(),	NULL,0,	NULL,0,	&fileTtl[0],MAX_PATH,	&fileExt[0],MAX_PATH) ;
	#else
	_tsplitpath		(pathName.c_str(),	NULL,	NULL,	&fileTtl[0],			&fileExt[0]) ;
	#endif
	return	tstring(fileTtl.c_str())+fileExt.c_str() ;
	}

inline	tstring	ise_GetFileExt (const tstring& pathName)
{
	tstring	fileExt ;	fileExt.resize(MAX_PATH,0) ;
	#if	(_MSC_VER >= 1400)
	_tsplitpath_s	(pathName.c_str(),	NULL,0,	NULL,0,	NULL,0,	&fileExt[0],MAX_PATH) ;
	#else
	_tsplitpath		(pathName.c_str(),	NULL,	NULL,	NULL,	&fileExt[0]) ;
	#endif
	fileExt = tstring(fileExt.c_str()) ;
	if (fileExt.length()>0) {
		fileExt = fileExt.substr(1) ;
		}
	return	fileExt ;
	}

//--------------------------------------------------------------------------------------------------
inline	size_t	ise_WC_GetLen	(const wchar_t ch)
{
	size_t	len = 0 ;
	if		(				ch <= 0x007f)	{	len = 1 ;	}	//	half
	else if (0xff61<= ch && ch <= 0xff9f)	{	len = 1 ;	}	//	kana
	else									{	len = 2 ;	}
	return	len ;
	}

inline	size_t	ise_GetLengthB	(const tstring& str)
{
	size_t	len = 0 ;
	#ifdef	_UNICODE
	{
		len = str.length() ;
		size_t	fullCount = 0 ;
		for (size_t index=0 ; index<len ; index++) {
			TCHAR	ch = str[index] ;
			if (::ise_WC_GetLen(ch) == 2)	{	fullCount++ ;	}
			}
		len += fullCount ;
		}
	#else
	{
		len = str.length() ;
		}
	#endif
	return	len ;
	}

inline	tstring	ise_CutString	(LPCTSTR str,const size_t lenW,const TCHAR sp=_T(' '))
{
	if (lenW == 0)		{	return	_T("") ;	}
	tstring	tmp = str ;
	tstring	newStr ;
	#ifdef	_UNICODE
	{
		size_t	fullCount=0 ;
		for (size_t index=0 ; index<lenW-fullCount && index<tmp.length() ; index++) {
			TCHAR	ch = tmp[index] ;
			if (::ise_WC_GetLen(ch) == 2)		{	fullCount++ ;	}
			newStr += ch ;
			}
		if (newStr.length()+fullCount > lenW)	{	newStr = tmp.substr(0,newStr.length()-1) ;	fullCount-- ;	}
		size_t cpyLen = newStr.length() ;
		if (sp != _T('\0')) {
			for (size_t spIndex=0 ; spIndex<lenW-(cpyLen+fullCount) ; spIndex++) {
				newStr += sp ;
				}
			}
		}
	#else
	{
		if (tmp.length() < lenW) {
			newStr = tmp ;
			}
		else {
			size_t	cpySize = 0 ;
			for (size_t index=0 ; index<lenW ; index++) {
				TCHAR	ch = tmp[index] ;
				if (_istlead(ch))	{	index++ ;	cpySize = index + 1 ;	}
				else				{				cpySize = index + 1 ;	}
				}
			if (cpySize > lenW)	{	newStr = tmp.substr(0,cpySize-2) ;	}
			else				{	newStr = tmp.substr(0,cpySize) ;	}
			}
		size_t	cpyLen = newStr.length() ;
		if (sp != _T('\0')) {
			for (size_t spIndex=0 ; spIndex<lenW-cpyLen ; spIndex++) {
				newStr += sp ;
				}
			}
		}
	#endif
	return	newStr ;
	}

//--------------------------------------------------------------------------------------------------
inline	tstring	ise_CutPathName (LPCTSTR srcName)
{
	#define	C_File_PathMax	100
	tstring	cutName = srcName ;
	if ((cutName.length() > C_File_PathMax)	||
		(::ise_GetLengthB(cutName) > C_File_PathMax)) {
		tstring	srcFold = ::ise_GetFileDir	(srcName) ;
		tstring	srcPage = ::ise_GetFileDir	(srcFold) ;
		tstring	docPage = ::ise_GetFileName	(srcPage) ;
		tstring	docFold = ::ise_GetFileName	(srcFold) ;
		tstring	docName = ::ise_GetFileName	(srcName) ;
		tstring	docPath ;
		if (docPage.length()		 > 30)	{	docPage = ::ise_CutString(docPage.c_str(),30) ;	}
		if (::ise_GetLengthB(docPage)> 30)	{	docPage = ::ise_CutString(docPage.c_str(),30) ;	}
		{
			if		(ise_GetLengthB(docPage) +	ise_GetLengthB(docFold) + ise_GetLengthB(docName) < C_File_PathMax) {
				docPath = docPage + _T("\\") +	docFold + _T("\\") +	docName ;
				}
			else if	(							ise_GetLengthB(docFold) + ise_GetLengthB(docName) < C_File_PathMax) {
				docPath = 						docFold + _T("\\") +	docName ;
				}
			else {
				docPath = 												docName ;
				}
			}
		cutName = docPath ;
		}
	return	cutName ;
	}

//--------------------------------------------------------------------------------------------------
tstring	ise_GetFolder	(
	const tstring&	base
,	const tstring&	sub1	//	=	_T("")
,	const tstring&	sub2	//	=	_T("")
,	const tstring&	sub3	//	=	_T("")
,	const tstring&	sub4		=	_T("")
	)
{
	tstring	folder ;
	{
		folder = ::ise_GetTempPath() ;
		}
	tstring	folderFull = folder ;
	{
		if (!base.empty()) {	folderFull +=			 base ;	}
		if (!sub1.empty()) {	folderFull += _T("\\")	+sub1 ;	}
		if (!sub2.empty()) {	folderFull += _T("\\")	+sub2 ;	}
		if (!sub3.empty()) {	folderFull += _T("\\")	+sub3 ;	}
		if (!sub4.empty()) {	folderFull += _T("\\")	+sub4 ;	}
		}
	#ifdef	Support_CreateFolder
	{
		if (!ise_CreateFolder(folderFull))	{	return	folder+base ;	}
		}
	#endif
	return	(folderFull+_T("\\")) ;
	}

//--------------------------------------------------------------------------------------------------
inline	tstring	ise_ChangePathSP	(const tstring& name)
{
	tstring	newName = name ;
	for (size_t index=0 ; index<newName.length() ; index++) {
		#ifdef	_MBCS
		if (  _istlead(newName[index]))	{	//	Skip 2-byte character
			index++ ;
			continue ;
			}
		#endif
		TCHAR	c = newName[index] ;
		switch	(c) {
			case	'\\' :		c = '_' ;		break ;
			case	'/' :		c = '_' ;		break ;
			case	':' :		c = '_' ;		break ;
			case	',' :		c = '.' ;		break ;
			case	';' :		c = '.' ;		break ;
			case	'*' :		c = 'x' ;		break ;
			case	'?' :		c = '_' ;		break ;
			case	'\"' :		c = '.' ;		break ;
			case	'<' :		c = '(' ;		break ;
			case	'>' :		c = ')' ;		break ;
			case	'|' :		c = '.' ;		break ;
			case	' ' :		c = '_' ;		break ;
			case	'\r' :		c = '.' ;		break ;
			case	'\n' :		c = '.' ;		break ;
			case	'\t' :		c = '.' ;		break ;
			default :			continue ;		break ;
			}
		newName[index] = c ;
		}
	return	newName ;
	}

//--------------------------------------------------------------------------------------------------
#define		ISE_CMF_Root	_T("Cache.tmp")
#define		ISE_CMF_Sub1	_T("CachMF")
#define		ISE_CMF_Sub2	_T("Other")
inline	tstring	ise_GetPathEMF	(LPCTSTR docPath)
{
	tstring	ext		= ::ise_GetFileExt	(docPath) ;
	tstring	docName = ::ise_CutPathName	(docPath) ;
	tstring	emfName	= ::ise_GetFolder	(ISE_CMF_Root,ISE_CMF_Sub1,ISE_CMF_Sub2,ext)
					+ ::ise_ChangePathSP(docName)	+ _T(".emf") ;
	return	emfName ;
	}

#endif
